{ ------------------------------------------------------------- }
{                                                               }
{   Name:           XCompCls.pas                                }
{   Version:        V 0.1                                       }
{   Author:         Erick I. Jimenez Alvarado                   }
{   To contact:     erickj@usa.net                              }
{                   http://members.xoom.com/erickj              }
{                                                               }
{   Description:    Based Class to Componenets                  }
{                                                               }
{   File Status:    COMPONETWARE ( like a FREEWARE but if you   }
{                   make a Componet or modify this please       }
{                   send or e-mail me this).                    }
{                                                               }
{   Last Upadated:  02/05/99                                    }
{                                                               }
{   History:                                                    }
{                                                               }
{    02/05/99   V0.1 -  First Version.                          }
{                                                               }
{   Finally:        Sorry my bad, bad english and thanks for    }
{                   use my component.                           }
{                                                               }
{ ------------------------------------------------------------- }

{.$DEFINE PRG_UNIT}

{$IFDEF PRG_UNIT}               { ---------------------- }

{$DEFINE PRG_INTERFACE}
{$DEFINE PRG_IMPLEMENTATION}


{$I XTools.Inc } { Directives for all controls }

Unit XCompCls;

interface

uses

 {$IFNDEF WIN32}
    WinTypes, WinProcs,                     { Default Units }
 {$ELSE}
    Windows,                                { Default Unit  }
 {$ENDIF}

    Classes, Controls, Graphics, Messages,  { Default Units }


    XObjects;


{$ENDIF} { // PRG_UNIT }


{$IFDEF PRG_INTERFACE}          { ---------------------- }


type

{
// --------------------------------------------------------
//  TEnhChangeLink Class
// --------------------------------------------------------
}
  TEnhContainer  = class;

  TEnhChangeLink = class(TObject)
  private

    FSender   : TEnhContainer;
    FOnChange : TNotifyEvent;

  public

    destructor  Destroy; override;
    procedure   Change; dynamic;

    property    OnChange: TNotifyEvent  read FOnChange write FOnChange;
    property    Sender  : TEnhContainer read FSender   write FSender;

  end;


{
// --------------------------------------------------------
//  TEnhComponent Class
// --------------------------------------------------------
}
  TEnhComponent = class(TComponent)

  private

    FOnChange  : TNotifyEvent;
    FLock      : Byte;

  protected

    procedure   GetChange(Sender : TObject); dynamic;
    procedure   Change; dynamic;

    procedure   Lock;
    procedure   UnLock(Value : Boolean);

    property    OnChange: TNotifyEvent read FOnChange write FOnChange;

  end;


{
// --------------------------------------------------------
//  TEnhContainer Class
// --------------------------------------------------------
}
  TEnhContainer   = class(TEnhComponent)

  private

    FClients   : TList;

  protected

    procedure   Change; override;

  public

    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;

    procedure   RegisterChanges(Value: TEnhChangeLink);
    procedure   UnRegisterChanges(Value: TEnhChangeLink);

    property    Clients: TList read FClients;

  end;



{
// --------------------------------------------------------
//  TFillBackground Class
// --------------------------------------------------------
}
  TFillBackground = class(TEnhContainer)

  private

    // FChangeLink : TEnhChangeLink;
    FBackground : TObjBackground;

    procedure   SetBackground(Value : TObjBackground);

  public

    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;

    procedure   Assign(Source : TPersistent); override;

  published

    property    Background : TObjBackground read FBackground write SetBackground;

  end;


{
// --------------------------------------------------------
//  TManagerContainer Class
// --------------------------------------------------------
}
  TManagerContainer  = class(TEnhContainer)

  private

    FFillBackground : TFillBackground;
//    FImageList      : TImageList;
//    FList           : TList;

    procedure   SetFillBackground(Value : TFillBackground);
//    procedure   SetImageList(Value : TImageList);

  protected

    procedure   GetChange(Sender : TObject); override;

  public

    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;

    property    FillBackground : TFillBackground read FFillBackground write SetFillBackground;
//    property    ImageList : TImageList read FImageList write SetImageList;

  end;


var
    ManagerContainer : TManagerContainer;

{$ENDIF} { // PRG_INTERFACE }

{$IFDEF PRG_UNIT}               { ---------------------- }

{
// --------------------------------------------------------
//
// --------------------------------------------------------
}
implementation
uses
     //GraphExt,
     SysUtlEx;

{$ENDIF} { // PRG_UNIT }

{$IFDEF PRG_IMPLEMENTATION}     { ---------------------- }


{
// --------------------------------------------------------
//  Default TEnhChangeLink Destructor
// --------------------------------------------------------
}
destructor TEnhChangeLink.Destroy;
begin

    if (Sender <> nil) then
        Sender.UnRegisterChanges(Self);

    inherited Destroy;
end;

{
// --------------------------------------------------------
//  Handle Change Event
// --------------------------------------------------------
}
procedure  TEnhChangeLink.Change;
begin
    if Assigned(FOnChange) then
        FOnChange(Sender);
end;


{
// --------------------------------------------------------
// --------------------------------------------------------
//  TEnhComponent Class
// --------------------------------------------------------
// --------------------------------------------------------
}

{
// --------------------------------------------------------
//  Lock Events
// --------------------------------------------------------
}
procedure TEnhComponent.Lock;
begin
    FLock := 1;
end;

{
// --------------------------------------------------------
//  UnLock Events
// --------------------------------------------------------
}
procedure TEnhComponent.UnLock(Value : Boolean);
begin

    FLock := 0;

    if (Value) then
        Change;                         { Notify change             }
end;

{
// --------------------------------------------------------
//  Create Change Event
// --------------------------------------------------------
}
procedure TEnhComponent.GetChange(Sender : TObject);
begin
    Change;
end;

{
// --------------------------------------------------------
//  Handle Change Event
// --------------------------------------------------------
}
procedure TEnhComponent.Change;
begin

    if (FLock = 1) then
        exit;

    { Execute Event Handler }
    if Assigned(FOnChange) then
        FOnChange(Self);
end;


{
// --------------------------------------------------------
// --------------------------------------------------------
//  TEnhContainer Class
// --------------------------------------------------------
// --------------------------------------------------------
}

{
// --------------------------------------------------------
//  Default Constructor
// --------------------------------------------------------
}
constructor TEnhContainer.Create(AOwner:TComponent);
begin
    inherited Create(AOwner);

    FClients := TList.Create;
end;

{
// --------------------------------------------------------
//  Default Destructor
// --------------------------------------------------------
}
destructor TEnhContainer.Destroy;
begin
    while (FClients.Count > 0) do
        UnRegisterChanges(TEnhChangeLink(FClients.Last));

    FClients.Free;

    inherited Destroy;
end;

{
// --------------------------------------------------------
//  Notify Change to client list
// --------------------------------------------------------
}
procedure  TEnhContainer.Change;
var
    I : Integer;

begin

    for I := 0 to FClients.Count - 1 do
        TEnhChangeLink(FClients[I]).Change;

    inherited Change;
end;

{
// --------------------------------------------------------
//  Add to client list
// --------------------------------------------------------
}
procedure  TEnhContainer.RegisterChanges(Value: TEnhChangeLink);
begin
    Value.Sender := Self;
    FClients.Add(Value);
end;

{
// --------------------------------------------------------
//  Remove from client list
// --------------------------------------------------------
}
procedure  TEnhContainer.UnRegisterChanges(Value : TEnhChangeLink);
var
    I : Integer;

begin
    for I := 0 to FClients.Count - 1 do
    begin
        if (FClients[i] = Value) then
        begin
            Value.Sender := nil;
            FClients.Delete(I);
            Break;
        end;
    end;
end;



{
// --------------------------------------------------------
// --------------------------------------------------------
//  TFillBackground Class
// --------------------------------------------------------
// --------------------------------------------------------
}

constructor TFillBackground.Create(AOwner: TComponent);
begin
    inherited Create(AOwner);                   { Parent Create         }

    FBackground := TObjBackground.Create;
    FBackground.OnChange := GetChange;          ///????
end;

{
// --------------------------------------------------------
//  Default Destructor
// --------------------------------------------------------
}
destructor TFillBackground.Destroy;
begin
    FBackground.OnChange := nil;
    FBackground.Free;

    inherited Destroy;
end;

{
// --------------------------------------------------------
//  Set Background object
// --------------------------------------------------------
}
procedure TFillBackground.SetBackground(Value: TObjBackground);
begin
    FBackground.Assign(Value);
end;

{
// --------------------------------------------------------
//  Set data object
// --------------------------------------------------------
}
procedure TFillBackground.Assign(Source: TPersistent);
begin

    if (Source = self) then
        exit;

    if (Source is TFillBackground) then
    begin
        FBackground.Assign(TFillBackground(Source).FBackground);
        exit;
    end;

    inherited Assign(Source);
end;



{
// --------------------------------------------------------
// --------------------------------------------------------
//  TManagerContainer Class
// --------------------------------------------------------
// --------------------------------------------------------
}
constructor TManagerContainer.Create(AOwner: TComponent);
begin
    inherited Create(AOwner);                   { Parent Create         }

    FFillBackground := TFillBackground.Create(Self);
    FFillBackground.OnChange := GetChange;          ///????

//    FImageList := TImageList.Create(self);
//    FImageList.OnChange := GetChange;

end;

{
// --------------------------------------------------------
//  Default Destructor
// --------------------------------------------------------
}
destructor TManagerContainer.Destroy;
begin
    FFillBackground.OnChange := nil;
    //FImageList.OnChange := nil;

    FFillBackground.Free;
    //FImageList.Free;

    inherited Destroy;
end;


{
// --------------------------------------------------------
//  Set Background
// --------------------------------------------------------
}
procedure TManagerContainer.SetFillBackground(Value: TFillBackground);
begin
    FFillBackground.Assign(Value);
end;

{
// --------------------------------------------------------
//  Set ImageList
// --------------------------------------------------------
}
//procedure TManagerContainer.SetImageList(Value:TImageList);
//begin
//    FImageList.Assign(Value);
//end;

{
// --------------------------------------------------------
//  Create Change Event
// --------------------------------------------------------
}
procedure TManagerContainer.GetChange(Sender:TObject);
begin
    if (csDesigning in ComponentState) and (Self <> ManagerContainer) then
    begin
        if (Sender is TFillBackground) then
            ManagerContainer.FFillBackground.Assign(FFillBackground);
//        if (Sender is TImageList)  then
//            ManagerContainer.ImageList.Assign(FImageList);
    end;

    inherited GetChange(Sender);
end;


{
// --------------------------------------------------------
// --------------------------------------------------------
//  Initialization
// --------------------------------------------------------
// --------------------------------------------------------
}
procedure  InitManager;
begin
    { Create Manager Container }
    ManagerContainer := TManagerContainer.Create(nil);
end;

procedure  FreeManager;
begin
    { Cleanup manager   }
    ManagerContainer.Free;
end;

{$ENDIF} { // PRG_IMPLEMENTATION }

{$IFDEF PRG_UNIT}               { ---------------------- }

initialization

    InitManager;

finalization
    FreeManager;

end.
{$ENDIF} { // PRG_UNIT }
