{ ------------------------------------------------------------- }
{                                                               }
{   Name:           XGradCls.pas                                }
{   Version:        V 0.6                                       }
{   Author:         Erick I. Jimenez Alvarado                   }
{   To contact:     erickj@usa.net                              }
{                   http://members.xoom.com/erickj              }
{                                                               }
{   Description:    Class to handle gradient functions          }
{                                                               }
{   File Status:    COMPONETWARE ( like a FREEWARE but if you   }
{                   make a Componet or modify this please       }
{                   send or e-mail me this).                    }
{                                                               }
{   Last Upadated:  01/21/99                                    }
{                                                               }
{   History:                                                    }
{                                                               }
{    01/04/99   V0.1 -  First relase version.                   }
{    01/07/99   V0.2 -  Add Waves efects, cool.                 }
{                       Put Compiler Directives to include      }
{                       Efects, Waves and Pallete to reduce     }
{                       and speed code.                         }
{    01/08/99   V0.3 -  Make Property Editor and Example.       }
{    01/19/99   V0.4 -  Rewrite all code enhanced all functions }
{    01/21/99   V0.5 -  Rewrite function Draw, correct few bugs }
{    01/22/99   V0.6 -  Add AutoFix property, that's cool.      }
{    01/25/99   V0.61-  Correct some bugs in DinamicPalette.    }
{                                                               }
{   Gratefulness: * Brad Stowers for all componets, great work  }
{                   thanks for all.                             }
{                   e-mail: bstowers@pobox.com                  }
{                   http://www.pobox.com/~bstowers/delphi/      }
{                                                               }
{   Finally:        Sorry my bad, bad english and thanks for    }
{                   use my component.                           }
{                                                               }
{ ------------------------------------------------------------- }

{.$DEFINE PRG_UNIT}

{$IFDEF PRG_UNIT}               { ---------------------- }

{$DEFINE PRG_INTERFACE}
{$DEFINE PRG_IMPLEMENTATION}

{$I XTools.Inc } { Directives for all controls }

Unit XGradCls;

interface

uses

 {$IFNDEF WIN32}
    WinTypes, WinProcs,             { Default Units }
 {$ELSE}
    Windows,                        { Default Unit  }
 {$ENDIF}  { // WIN32 }

  Classes, Graphics,                { Default Units }

  GraphExt;                         { My Unit       }

{$ENDIF} { // PRG_UNIT }


{$IFDEF PRG_INTERFACE}          { ---------------------- }

const

  Min_Grad_Labels = 4;
  Max_Grad_Labels = LowMaxPaletteSize;

  Def_Grad_Labels = 16;

{$IFDEF LIKEWAVE }
  Min_Waves = 1;
  Max_Waves = 64;
{$ENDIF}  { // LIKEWAVE }


type

 {$IFDEF LIKEALLEFECTS}

  { Gradient FX }
  TEnhGradientSet = (gfxSwap,
                    {$IFDEF LIKEPALETTE } gfxDinamicPalette, {$ENDIF}
                     gfxAutoFix);
  TEnhGradientFx  = set of TEnhGradientSet;

 {$ENDIF}  { // LIKEALLEFECTS }


{
// --------------------------------------------------------
//  Gradient Class
// --------------------------------------------------------
}
  TObjGradient = Class(TPersistent)

  private

    FLock       : Byte;                 { Lock events                   }

    FOnChange   : TNotifyEvent;         { handle for change on property }

    FBeginColor : TColor;               { Begin Color                   }
    FEndColor   : TColor;               { End Color                     }

  {$IFDEF LIKEALLEFECTS}
    FEfects     : TEnhGradientFx;       { Control Efect                 }
    FStyle      : TGradientStyle;       { Style of Gradient             }
  {$ENDIF}  { // LIKEALLEFECTS }

  {$IFDEF LIKEWAVE }
    FWaves      : Byte;                 { Number of waves               }
  {$ENDIF}  { // LIKEWAVE }

  {$IFDEF LIKEPALETTE }
    FPalette    : HPalette;             { Palette of colors             }
  {$ENDIF}  { // LIKEPALETTE }

    FColorList  : TColorList;           { Array to put RGB Colors       }

    procedure   SetBeginColor(Value: TColor);
    procedure   SetEndColor(Value: TColor);
    function    GetColorLevels: Byte;
    procedure   SetColorLevels(Value: Byte);

  {$IFDEF LIKEALLEFECTS}
    procedure   SetStyle(Value: TGradientStyle);
    procedure   SetEfects(Value: TEnhGradientFx);
  {$ENDIF}  { // LIKEALLEFECTS }

  {$IFDEF LIKEWAVE }
    procedure   SetWaves(Value: Byte);
  {$ENDIF}  { // LIKEWAVE }

  {$IFDEF LIKEPALETTE }
//    function    GetPalette:HPalette;
  {$ENDIF}  { // LIKEPALETTE }

  protected

  {$IFDEF LIKEPALETTE }
    procedure   DestroyPalette;
  {$ENDIF}  { // LIKEPALETTE }

    procedure   Lock;
    procedure   UnLock(Value:Boolean);

  public

    constructor Create; dynamic;
    destructor  Destroy; override;

    procedure   PaintDC(FHandle: HDC; aRect: TRect);       // dynamic;

    procedure   Assign(Source: TPersistent); override;

    procedure   Invalidate; // dynamic;

  {$IFDEF LIKEPALETTE }
    function    PutPalette(DC : HDC) : HPalette;

    property    Palette     : HPalette read FPalette;
  {$ENDIF}  { // LIKEPALETTE }

    property    OnChange: TNotifyEvent read FOnChange write FOnChange;

  published

    property    BeginColor  : TColor  read FBeginColor   write SetBeginColor
                            default clBlack;
    property    EndColor    : TColor  read FEndColor     write SetEndColor
                            default clNavy;
    property    ColorLevels : Byte    read GetColorLevels write SetColorLevels
                            default Def_Grad_Labels;

  {$IFDEF LIKEALLEFECTS}

  {$IFDEF LIKEWAVE }
    property    Waves       : Byte    read FWaves        write SetWaves default 1;
  {$ENDIF}  { // LIKEWAVE }

    property    Efects      : TEnhGradientFx read FEfects  write SetEfects
                            default [];
    property    Style       : TGradientStyle read FStyle   write SetStyle
                            default gsHorizontal;
  {$ENDIF}  { // LIKEALLEFECTS }

  end;


{$ENDIF} { // PRG_INTERFACE }


{$IFDEF PRG_UNIT}               { ---------------------- }

{
// --------------------------------------------------------
//
// --------------------------------------------------------
}
Implementation

Uses

    SysUtils;                       { Default Units }

{$ENDIF} { // PRG_UNIT }


{$IFDEF PRG_IMPLEMENTATION}     { ---------------------- }


{
// --------------------------------------------------------
//  Default Constructor
// --------------------------------------------------------
}
constructor TObjGradient.Create;
begin

    inherited Create;                   { Parent Create                  }

    { Set Properties }
    FBeginColor := clBlack;             { Default Begin Color            }
    FEndColor   := clNavy;              { Default End Color              }

  {$IFDEF LIKEALLEFECTS}

    FEfects     := [];                  { Correct Gradient Numbers       }
    FStyle      := gsHorizontal;        { Default style of gradient      }

  {$IFDEF LIKEWAVE }
    FWaves      := Min_Waves;           { Default Number of Waves        }
  {$ENDIF}  { // LIKEWAVE }

  {$IFDEF LIKEPALETTE }
    FPalette    := 0;                   { Handle of Palette = 0          }
  {$ENDIF}  { // LIKEPALETTE }

  {$IFDEF LIKEDCOLORS}
    FColorList  := TColorList.Create(Def_Grad_Labels);  { Create List of colors }

  {$ELSE}

    FColorList.Items := Def_Grad_Labels;     { Default Number of Steps Colors }
    GetMem(FColorList.List, Max_Grad_Labels * SizeOf(TRGBMap));

  {$ENDIF}  { // LIKEDCOLORS }


  {$ENDIF}  { // LIKEALLEFECTS }

    FPalette := 0;
    FLock    := 0;                      { Unlock Refresh and draw   }

    { Make list of Gradient Colors }
//    CreateGradientColors(TRGBMap(ColorToRGB(clBlack)),TRGBMap(ColorToRGB(clNavy)),
    CreateGradientColors(TRGBMap(RGBBlackColor),RGBNavyColor,
                        FColorList.List, Def_Grad_Labels
                        {$IFDEF LIKEPALETTE},False {$ENDIF} );
end;

{
// --------------------------------------------------------
//  Default Destructor
// --------------------------------------------------------
}
destructor TObjGradient.Destroy;
begin

  {$IFDEF LIKEPALETTE }
    DestroyPalette;                     { Delete PAlette                 }
  {$ENDIF}  { // LIKEPALETTE }

  {$IFDEF LIKEDCOLORS}
    FColorList.Free;                    { Relase Array of colors         }
  {$ELSE}
    FreeMem(FColorList.List,Max_Grad_Labels * SizeOf(TRGBMap));
    FList   := nil;
  {$ENDIF}  { // LIKEDCOLORS }

    inherited Destroy;
end;


{
// --------------------------------------------------------
// Set Begin Color
// --------------------------------------------------------
}
procedure TObjGradient.SetBeginColor(Value: TColor);
begin
    if (Value <> FBeginColor) then
    begin
        FBeginColor := Value;
        Invalidate;                             { Notify change         }
    end;
end;

{
// --------------------------------------------------------
// Set End Color
// --------------------------------------------------------
}
procedure TObjGradient.SetEndColor(Value: TColor);
begin
    if (Value <> FEndColor) then
    begin
        FEndColor := Value;
        Invalidate;                             { Notify change         }
    end;
end;

{
// --------------------------------------------------------
// Set ColorLevels
// --------------------------------------------------------
}
procedure TObjGradient.SetColorLevels(Value: Byte);
begin
    if (Value <> FColorList.Items) then
    begin
        { Check for min colors levels }
        if (Value < Min_Grad_Labels) then
            Value := Min_Grad_Labels;

        { Check for max colors levels }
        if (Value > Max_Grad_Labels) then
            Value := Max_Grad_Labels;

      {$IFDEF LIKEDCOLORS}
        FColorList.Resize(Value);               { Resize the array          }
      {$ELSE}
        FColorList.Items := Value;              { Put Number of Levels      }
      {$ENDIF}  { // LIKEDCOLORS }

        Invalidate;                             { Notify change         }
    end;
end;

{
// --------------------------------------------------------
// Get ColorLevels
// --------------------------------------------------------
}
function TObjGradient.GetColorLevels: Byte;
begin
    Result := FColorList.Items;             { Put Number of Levels      }
end;


{
// --------------------------------------------------------
//  Copy all gradient properties to self
// --------------------------------------------------------
}
procedure TObjGradient.Assign(Source:TPersistent);
begin

    if (Source = nil) then
    Begin
      {$IFDEF DEBUGCODE}
        raise Exception.Create('TObjGradient::Assign: Invalid Pointer');
      {$ENDIF} { // DEBUGCODE }
        exit;
    End;

    if (Source = self) then
        exit;

    { Check if correct class }
    if (Source is TObjGradient) then
    begin

        FLock := 1;                   { Lock Events           }

      try

        FBeginColor := TObjGradient(Source).FBeginColor;
        FEndColor   := TObjGradient(Source).FEndColor;

        { Use property to assign value and Reset List of colors }
        ColorLevels := TObjGradient(Source).ColorLevels;

      {$IFDEF LIKEALLEFECTS}

      {$IFDEF LIKEWAVE }
        FWaves      := TObjGradient(Source).FWaves;
      {$ENDIF}  { // LIKEWAVE }

        FStyle      := TObjGradient(Source).FStyle;
        FEfects     := TObjGradient(Source).FEfects;

      {$ENDIF}  { // LIKEALLEFECTS }

      finally

        FLock := 0;                     { UnLock Events         }
        Invalidate;                     { Notify change             }

      end;

        Exit;

    end;

    { Call inherited }
    inherited Assign(Source);
end;


{
// --------------------------------------------------------
//  Lock Events
// --------------------------------------------------------
}
procedure TObjGradient.Lock;
begin
    FLock := 1;
end;

{
// --------------------------------------------------------
//  UnLock Events
// --------------------------------------------------------
}
procedure TObjGradient.UnLock(Value:Boolean);
begin

    FLock := 0;

    if (Value) then
        Invalidate;                         { Notify change             }
end;

{
// --------------------------------------------------------
//  Refresh Data and call Event OnChange
// --------------------------------------------------------
}
procedure TObjGradient.Invalidate;
var
    LoColor, HiColor, TmpColor: TRGBMap;
begin

    { Check if lock procedure }
    if (FLock = 1) then
        Exit;


    { Convert TColor to RGB Color }
    LoColor.RGBVal := ColorToRGB(FBeginColor);
    HiColor.RGBVal := ColorToRGB(FEndColor);

  {$IFDEF LIKEALLEFECTS}

    { Check if Swap colors  }
    if (gfxSwap in FEfects) then
    begin
        TmpColor:= LoColor;
        LoColor := HiColor;
        HiColor := tmpColor;
    end;

  {$IFDEF LIKEPALETTE }
    DestroyPalette;                         { Delete current palette    }
  {$ENDIF}  { // LIKEPALETTE }


  {$ENDIF} { // LIKEALLEFECTS }

    { Make list of Gradient Colors }
    CreateGradientColors(LoColor,HiColor,FColorList.List,FColorList.Items
          {$IFDEF LIKEPALETTE},(gfxDinamicPalette in FEfects) {$ENDIF} );


  {$IFDEF LIKEPALETTE }

    { check for dinamic palette }
    if (gfxDinamicPalette in FEfects) then
        FPalette := CreateListPalette(FColorList.Items, FColorList.List );

  {$ENDIF}  { // LIKEPALETTE }

    { Call event on object change }
    if Assigned(FOnChange) then
        FOnChange(Self);

end;

{
// --------------------------------------------------------
// Draw Gradient Object
// --------------------------------------------------------
}
procedure TObjGradient.PaintDC(FHandle : HDC; aRect: TRect);

{$IFDEF LIKEALLEFECTS}

Var
    I,J  : Integer;

{$ENDIF}  { // LIKEALLEFECTS }

begin

  {$IFNDEF DEBUGCODE}

    if (FLock = 1) or (FHandle = 0) or IsRectEmpty(aRect) then
        Exit;

  {$ELSE}

    if (FLock = 1) or IsRectEmpty(aRect) then
        Exit;


    if (FHandle = 0)  then
    Begin
        raise Exception.Create('TObjGradient::PaintDC: Handle not valid');
        Exit;
    End;

  {$ENDIF} { // DEBUGCODE }

    { Check if color list Exist }
    if (FColorList.Items = 0) then // or (FColorList.List = nil) then
    begin
     {$IFDEF DEBUGCODE}
        raise Exception.Create('TObjGradient::PaintDC: List of Colors Error');
     {$ENDIF} { // DEBUGCODE }
        Exit;
    end;

    { Adjust }

  {$IFDEF LIKEALLEFECTS}

    { Check if self correct color levels }
    if (gfxAutoFix in FEfects) then
    Begin
        With aRect do
        Begin
            I := (Right  - Left) Shr 2;
            J := (Bottom - Top)  Shr 2;

            if (FStyle = gsVertical) then
                I := J
            else if not (FStyle = gsHorizontal) then
            Begin
                if ( I < J) then
                    I := J;

                if (FStyle = gsSquare) then
                    I := Round(I / 1.5)
                    //I := (I Shr 1) + 1;
            end;
        end;

        ColorLevels := I + 1;
    End;

//  {$IFDEF LIKEPALETTE}
//    PutPalette();
//
//  {$ENDIF}  { // LIKEPALETTE }


  {$IFDEF LIKEWAVE}

    if (FWaves > 1) then
        { Ejecute real draw function }
        DrawGradientW(FHandle, aRect, FColorList.List, FColorList.Items,FWaves,FStyle)
    else

  {$ENDIF}  { // LIKEWAVE }

  {$ENDIF}  { // LIKEALLEFECTS }

    { Ejecute real draw function }
    DrawGradient(FHandle, aRect, FColorList.List, FColorList.Items,FStyle);

end;


{$IFDEF LIKEALLEFECTS}

{
// --------------------------------------------------------
// Set StyleColor and refresh data
// --------------------------------------------------------
}
procedure TObjGradient.SetStyle(Value: TGradientStyle);
begin
    if (Value <> FStyle) then
    begin
        FStyle := Value;
        Invalidate;                             { Notify change         }
    end;
end;

{
// --------------------------------------------------------
// Set Efects and refresh data
// --------------------------------------------------------
}
procedure TObjGradient.SetEfects(Value: TEnhGradientFx);
begin
    if (Value <> FEfects) then
    begin
        FEfects := Value;
        Invalidate;                             { Notify change         }
    end;
end;

{$ENDIF}  { // LIKEALLEFECTS }


{$IFDEF LIKEWAVE }
{
// --------------------------------------------------------
// Set Waves
// --------------------------------------------------------
}
procedure TObjGradient.SetWaves(Value: Byte);
begin
    if (Value <> FWaves) then
    begin
        { Correct Min Value             }
        if (Value < Min_Waves) then
            Value := Min_Waves;

        { Correct Max Value             }
        if (Value > Max_Waves) then
            Value := Max_Waves;

        FWaves := Value;
        Invalidate;                             { Notify change         }
    end;
end;
{$ENDIF}  { // LIKEWAVE }


{$IFDEF LIKEPALETTE }
{
// --------------------------------------------------------
// Set Gradient Palette
// --------------------------------------------------------
}
function TObjGradient.PutPalette(DC : HDC): HPalette;
begin
    Result := 0;

    if (gfxDinamicPalette in FEfects) then
    begin

        {  Check for low color ressolution   }
        if (GetSystemColorBitDepth < 8) then
        begin
            Exclude(FEfects,gfxDinamicPalette);     { Reset Palette         }
            Invalidate;
        end
        else
        begin
            Result := SelectPalette(HDC(Pointer(DC)),FPalette, False);
            RealizePalette(HDC(Pointer(DC)));
        end;

    end;

end;

{
// --------------------------------------------------------
//  Destroy Palette handle
// --------------------------------------------------------
}
procedure TObjGradient.DestroyPalette;
begin
    if (FPalette <> 0) then
    Begin
        if (DeleteObject(FPalette)) then
            FPalette := 0;
    End;
end;

{$ENDIF}  { // LIKEPALETTE }

{$ENDIF} { // PRG_IMPLEMENTATION }

{$IFDEF PRG_UNIT}               { ---------------------- }
end.
{$ENDIF} { // PRG_UNIT }
